@@ -9,8 +9,6 @@ use std::time::{Duration, Instant};
 pub struct ClientSocketConnection {
     pub room: Recipient<MessageFromClient>,
     pub client_id: ClientId,
-    pub room_id: String,
-    pub ip: String,
     pub last_seen: Instant,
     pub heartbeat_interval: Duration,
     pub heartbeat_timeout: Duration,
@@ -23,8 +21,6 @@ impl ClientSocketConnection {
             if Instant::now() - act.last_seen > act.heartbeat_timeout {
                 tracing::warn!(
                     client_id=?act.client_id,
-                    ip=%act.ip,
-                    room_id=%act.room_id,
                     "Stopping ClientSocketConnection because heartbeat not responded.",
                 );
                 act.close(ctx);
@@ -42,10 +38,7 @@ impl ClientSocketConnection {
             .do_send(MessageFromClient::Disconnect(self.client_id))
             .is_err()
         {
-            tracing::warn!(
-                room_id=%self.room_id,
-                "Could not send Disconnect message before closing room",
-            );
+            tracing::warn!("Could not send Disconnect message before closing room",);
         }
 
         ctx.stop();
@@ -82,10 +75,7 @@ impl StreamHandler<Result<ws::Message, ws::ProtocolError>> for ClientSocketConne
                     data: MessageData::String(text.to_string()),
                 };
                 if self.room.do_send(message).is_err() {
-                    tracing::warn!(
-                        room_id=%self.room_id,
-                        "Error forwarding message to service",
-                    );
+                    tracing::warn!("Error forwarding message to service",);
                 }
             }
             Ok(ws::Message::Binary(data)) => {
@@ -94,17 +84,12 @@ impl StreamHandler<Result<ws::Message, ws::ProtocolError>> for ClientSocketConne
                     data: MessageData::Binary(data.to_vec()),
                 };
                 if self.room.do_send(message).is_err() {
-                    tracing::warn!(
-                        room_id=%self.room_id,
-                        "Error forwarding binary message to service",
-                    );
+                    tracing::warn!("Error forwarding binary message to service",);
                 }
             }
             Ok(ws::Message::Close(_)) => {
                 tracing::info!(
                     client_id=?self.client_id,
-                    ip=%self.ip,
-                    room_id=%self.room_id,
                     "User has disconnected from room",
                 );
 
